var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import * as React from 'react';
import { useUniqueId } from '@workday/canvas-kit-react-common';
import Tab from './Tab';
import TabList from './TabList';
import TabPanel from './TabPanel';
var noop = function () { };
var TabsStateContext = React.createContext({
    id: '',
    activeTab: '',
    setActiveTab: function () { return ({}); },
    intentTab: '',
    setIntentTab: noop,
    resetIntentTab: noop,
    selectedTabRect: null,
    setSelectedTabRect: function () { return ({}); },
    registerTab: function () { return ''; },
    unregisterTab: noop,
    registerPanel: function () { return ''; },
    unregisterPanel: noop,
    programmaticFocusRef: { current: false },
});
export var useTab = function () { return React.useContext(TabsStateContext); };
var Tabs = function (_a) {
    var children = _a.children, _b = _a.initialTab, initialTab = _b === void 0 ? '' : _b, onTabChange = _a.onTabChange, elemProps = __rest(_a, ["children", "initialTab", "onTabChange"]);
    var id = useUniqueId();
    var _c = React.useState(initialTab), activeTab = _c[0], setActiveTabState = _c[1];
    var _d = React.useState(initialTab), intentTab = _d[0], setIntentTabState = _d[1];
    var _e = React.useState(null), selectedTabRect = _e[0], setSelectedTabRect = _e[1];
    var tabsRef = React.useRef([]);
    var panelsRef = React.useRef([]);
    var programmaticFocusRef = React.useRef(false);
    var onTabChangeRef = React.useRef(onTabChange);
    var registerTab = React.useCallback(function (element, name) {
        if (!name) {
            name = String(tabsRef.current.length);
        }
        tabsRef.current.push({ element: element, name: name });
        return name;
    }, [tabsRef]);
    var unregisterTab = React.useCallback(function (element) {
        tabsRef.current = tabsRef.current.filter(function (tab) { return tab.element !== element; });
    }, [tabsRef]);
    var registerPanel = React.useCallback(function (name) {
        if (!name) {
            name = String(panelsRef.current.length);
        }
        panelsRef.current.push(name);
        return name;
    }, [panelsRef]);
    var unregisterPanel = React.useCallback(function (name) {
        panelsRef.current = panelsRef.current.filter(function (tab) { return tab !== name; });
    }, [panelsRef]);
    var setActiveTab = React.useCallback(function (name) {
        var _a;
        setIntentTabState(name);
        setActiveTabState(name);
        (_a = onTabChangeRef.current) === null || _a === void 0 ? void 0 : _a.call(onTabChangeRef, name);
    }, [setActiveTabState, setIntentTabState, onTabChangeRef]);
    var setIntentTab = React.useCallback(function (value) {
        programmaticFocusRef.current = true;
        var nextIndex = 0;
        if (value === 'first') {
            nextIndex = 0;
        }
        else if (value === 'last') {
            nextIndex = tabsRef.current.length - 1;
        }
        else {
            nextIndex =
                tabsRef.current.findIndex(function (tab) { return tab.name === (intentTab || activeTab); }) +
                    (value === 'next' ? 1 : -1);
            if (nextIndex < 0) {
                nextIndex = tabsRef.current.length - 1;
            }
            else if (nextIndex >= tabsRef.current.length) {
                nextIndex = 0;
            }
        }
        setIntentTabState(tabsRef.current[nextIndex].name);
        tabsRef.current[nextIndex].element.focus();
    }, [activeTab, intentTab]);
    var resetIntentTab = React.useCallback(function () {
        if (programmaticFocusRef.current) {
            return;
        }
        setIntentTabState(activeTab);
    }, [setIntentTabState, activeTab, programmaticFocusRef]);
    React.useLayoutEffect(function () {
        var _a, _b;
        if (!initialTab) {
            setActiveTabState(((_a = tabsRef.current[0]) === null || _a === void 0 ? void 0 : _a.name) || '');
            setIntentTabState(((_b = tabsRef.current[0]) === null || _b === void 0 ? void 0 : _b.name) || '');
        }
    }, []);
    var value = {
        id: id,
        activeTab: activeTab,
        setActiveTab: setActiveTab,
        intentTab: intentTab,
        setIntentTab: setIntentTab,
        resetIntentTab: resetIntentTab,
        selectedTabRect: selectedTabRect,
        setSelectedTabRect: setSelectedTabRect,
        registerTab: registerTab,
        unregisterTab: unregisterTab,
        registerPanel: registerPanel,
        unregisterPanel: unregisterPanel,
        programmaticFocusRef: programmaticFocusRef,
    };
    return (React.createElement(TabsStateContext.Provider, { value: value },
        React.createElement("div", __assign({}, elemProps), children)));
};
Tabs.List = TabList;
Tabs.Item = Tab;
Tabs.Panel = TabPanel;
export default Tabs;
